@@ -16,6 +16,8 @@ ChatDocumentHandler::ChatDocumentHandler(QObject *parent)
     : QObject(parent)
     , m_document(nullptr)
     , m_cursorPosition(-1)
+    , m_selectionStart(-1)
+    , m_selectionEnd(-1)
 {
 }
 
@@ -123,7 +125,7 @@ void ChatDocumentHandler::setRoom(NeoChatRoom *room)
     Q_EMIT roomChanged();
 }
 
-QVariantMap ChatDocumentHandler::getAutocompletionInfo()
+QVariantMap ChatDocumentHandler::getAutocompletionInfo(bool isAutocompleting)
 {
     QTextCursor cursor = textCursor();
 
@@ -133,9 +135,6 @@ QVariantMap ChatDocumentHandler::getAutocompletionInfo()
             {"type", AutoCompletionType::Ignore},
         };
     }
-    if (m_cursorPosition != m_autoCompleteBeginPosition && m_cursorPosition != m_autoCompleteEndPosition) {
-        // we moved our cursor, so cancel autocompletion
-    }
 
     QString text = cursor.block().text();
     QString textBeforeCursor = text;
@@ -167,10 +166,17 @@ QVariantMap ChatDocumentHandler::getAutocompletionInfo()
             };
         }
 
-        return QVariantMap{
-            {"keyword", autoCompletePrefix},
-            {"type", AutoCompletionType::Emoji},
-        };
+        if (!isAutocompleting) {
+            return QVariantMap{
+                {"keyword", autoCompletePrefix},
+                {"type", AutoCompletionType::Emoji},
+            };
+        } else {
+            return QVariantMap{
+                {"type", AutoCompletionType::Ignore},
+                {"keyword", autoCompletePrefix},
+            };
+        }
     }
 
     return QVariantMap{
