@@ -34,7 +34,12 @@
 
 #include <signal.h>
 
+#ifdef QUOTIENT_07
 #include "accountregistry.h"
+#else
+#include "neochataccountregistry.h"
+#endif
+
 #include "csapi/account-data.h"
 #include "csapi/content-repo.h"
 #include "csapi/joining.h"
@@ -60,7 +65,6 @@ using namespace Quotient;
 
 Controller::Controller(QObject *parent)
     : QObject(parent)
-    , m_mgr(new QNetworkConfigurationManager(this))
 {
     Connection::setRoomType<NeoChatRoom>();
     Connection::setUserType<NeoChatUser>();
@@ -115,8 +119,6 @@ Controller::Controller(QObject *parent)
         sigaction(sig, &sa, nullptr);
     }
 #endif
-
-    connect(m_mgr, &QNetworkConfigurationManager::onlineStateChanged, this, &Controller::isOnlineChanged);
 }
 
 Controller::~Controller()
@@ -262,31 +264,48 @@ void Controller::invokeLogin()
             id = accountId;
         }
         if (!account.homeserver().isEmpty()) {
-            auto accessToken = loadAccessTokenFromKeyChain(account);
-
-            auto connection = new Connection(account.homeserver());
-            connect(connection, &Connection::connected, this, [this, connection, id] {
-                connection->loadState();
-                addConnection(connection);
-                if (connection->userId() == id) {
-                    setActiveConnection(connection);
-                    connectSingleShot(connection, &Connection::syncDone, this, &Controller::initiated);
-                }
-            });
-            connect(connection, &Connection::loginError, this, [this, connection](const QString &error, const QString &) {
-                if (error == "Unrecognised access token") {
-                    Q_EMIT errorOccured(i18n("Login Failed: Access Token invalid or revoked"));
-                    logout(connection, false);
+            auto accessTokenLoadingJob = loadAccessTokenFromKeyChain(account);
+            connect(accessTokenLoadingJob, &QKeychain::Job::finished, this, [accountId, id, this, accessTokenLoadingJob](QKeychain::Job *) {
+                AccountSettings account{accountId};
+                QString accessToken;
+                if (accessTokenLoadingJob->error() == QKeychain::Error::NoError) {
+                    accessToken = accessTokenLoadingJob->binaryData();
                 } else {
-                    Q_EMIT errorOccured(i18n("Login Failed: %1", error));
-                    logout(connection, true);
+                    // No access token from the keychain, try token file
+                    // TODO FIXME this code is racy since the file might have
+                    // already been removed. But since the other code do a blocking
+                    // dbus call, the probability are not high that it will happen.
+                    // loadAccessTokenFromFile is also mostly legacy nowadays
+                    accessToken = loadAccessTokenFromFile(account);
+                    if (accessToken.isEmpty()) {
+                        return;
+                    }
                 }
-                Q_EMIT initiated();
+
+                auto connection = new Connection(account.homeserver());
+                connect(connection, &Connection::connected, this, [this, connection, id] {
+                    connection->loadState();
+                    addConnection(connection);
+                    if (connection->userId() == id) {
+                        setActiveConnection(connection);
+                        connectSingleShot(connection, &Connection::syncDone, this, &Controller::initiated);
+                    }
+                });
+                connect(connection, &Connection::loginError, this, [this, connection](const QString &error, const QString &) {
+                    if (error == "Unrecognised access token") {
+                        Q_EMIT errorOccured(i18n("Login Failed: Access Token invalid or revoked"));
+                        logout(connection, false);
+                    } else {
+                        Q_EMIT errorOccured(i18n("Login Failed: %1", error));
+                        logout(connection, true);
+                    }
+                    Q_EMIT initiated();
+                });
+                connect(connection, &Connection::networkError, this, [this](const QString &error, const QString &, int, int) {
+                    Q_EMIT errorOccured(i18n("Network Error: %1", error));
+                });
+                connection->assumeIdentity(account.userId(), accessToken, account.deviceId());
             });
-            connect(connection, &Connection::networkError, this, [this](const QString &error, const QString &, int, int) {
-                Q_EMIT errorOccured(i18n("Network Error: %1", error));
-            });
-            connection->assumeIdentity(account.userId(), accessToken, account.deviceId());
         }
     }
     if (accounts.isEmpty()) {
@@ -309,48 +328,57 @@ QByteArray Controller::loadAccessTokenFromFile(const AccountSettings &account)
     return {};
 }
 
-QByteArray Controller::loadAccessTokenFromKeyChain(const AccountSettings &account)
+QKeychain::ReadPasswordJob *Controller::loadAccessTokenFromKeyChain(const AccountSettings &account)
 {
-    QKeychain::Error error;
-    QString errorString;
-    do {
-        qDebug() << "Reading access token from the keychain for" << account.userId();
-        QKeychain::ReadPasswordJob job(qAppName());
-        job.setAutoDelete(false);
-        job.setKey(account.userId());
-        QEventLoop loop;
-        QKeychain::ReadPasswordJob::connect(&job, &QKeychain::Job::finished, &loop, &QEventLoop::quit);
-        job.start();
-        loop.exec();
+    qDebug() << "Reading access token from the keychain for" << account.userId();
+    auto job = new QKeychain::ReadPasswordJob(qAppName(), this);
+    job->setKey(account.userId());
 
-        if (job.error() == QKeychain::Error::NoError) {
-            return job.binaryData();
+    // Handling of errors
+    connect(job, &QKeychain::Job::finished, this, [this, &account, job]() {
+        if (job->error() == QKeychain::Error::NoError) {
+            return;
         }
-        Q_EMIT globalErrorOccured(i18n("Unable to read access token"), i18n("Please make sure that the keychain is opened."));
-        error = job.error();
-        errorString = job.errorString();
-    } while (error == QKeychain::Error::OtherError);
-
-    qWarning() << "Could not read the access token from the keychain:" << errorString;
-    // no access token from the keychain, try token file
-    auto accessToken = loadAccessTokenFromFile(account);
-    if (error == QKeychain::Error::EntryNotFound) {
-        if (!accessToken.isEmpty()) {
-            qDebug() << "Migrating the access token from file to the keychain for " << account.userId();
-            bool removed = false;
-            bool saved = saveAccessTokenToKeyChain(account, accessToken);
-            if (saved) {
-                QFile accountTokenFile{accessTokenFileName(account)};
-                removed = accountTokenFile.remove();
-            }
-            if (!(saved && removed)) {
-                qDebug() << "Migrating the access token from the file to the keychain "
-                            "failed";
+        if (job->error() == QKeychain::Error::EntryNotFound) {
+            // no access token from the keychain, try token file
+            auto accessToken = loadAccessTokenFromFile(account);
+            if (!accessToken.isEmpty()) {
+                qDebug() << "Migrating the access token from file to the keychain for " << account.userId();
+                bool removed = false;
+                bool saved = saveAccessTokenToKeyChain(account, accessToken);
+                if (saved) {
+                    QFile accountTokenFile{accessTokenFileName(account)};
+                    removed = accountTokenFile.remove();
+                }
+                if (!(saved && removed)) {
+                    qDebug() << "Migrating the access token from the file to the keychain "
+                                "failed";
+                }
+                return;
             }
         }
-    }
 
-    return accessToken;
+        switch (job->error()) {
+        case QKeychain::EntryNotFound:
+            Q_EMIT globalErrorOccured(i18n("Access token wasn't found"), i18n("Maybe it was deleted?"));
+            break;
+        case QKeychain::AccessDeniedByUser:
+        case QKeychain::AccessDenied:
+            Q_EMIT globalErrorOccured(i18n("Access to keychain was denied."), i18n("Please allow NeoChat to read the access token"));
+            break;
+        case QKeychain::NoBackendAvailable:
+            Q_EMIT globalErrorOccured(i18n("No keychain available."), i18n("Please install a keychain, e.g. KWallet or GNOME keyring on Linux"));
+            break;
+        case QKeychain::OtherError:
+            Q_EMIT globalErrorOccured(i18n("Unable to read access token"), job->errorString());
+            break;
+        default:
+            break;
+        }
+    });
+    job->start();
+
+    return job;
 }
 
 bool Controller::saveAccessTokenToFile(const AccountSettings &account, const QByteArray &accessToken)
@@ -388,16 +416,6 @@ bool Controller::saveAccessTokenToKeyChain(const AccountSettings &account, const
     return true;
 }
 
-void Controller::playAudio(const QUrl &localFile)
-{
-    auto player = new QMediaPlayer;
-    player->setMedia(localFile);
-    player->play();
-    connect(player, &QMediaPlayer::stateChanged, [player] {
-        player->deleteLater();
-    });
-}
-
 void Controller::changeAvatar(Connection *conn, const QUrl &localFile)
 {
     auto job = conn->uploadFile(localFile.toLocalFile());
@@ -547,9 +565,26 @@ void Controller::setActiveConnection(Connection *connection)
     if (connection == m_connection) {
         return;
     }
+    if (m_connection != nullptr) {
+        disconnect(connection, &Connection::syncError, this, nullptr);
+    }
     m_connection = connection;
     if (connection != nullptr) {
         NeoChatConfig::self()->setActiveConnection(connection->userId());
+        connect(connection, &Connection::networkError, this, [this](QString message, QString details, int retriesTaken, int nextRetryInMilliseconds) {
+            if (!m_isOnline) {
+                return;
+            }
+            m_isOnline = false;
+            Q_EMIT isOnlineChanged(false);
+        });
+        connect(connection, &Connection::syncDone, this, [this] {
+            if (m_isOnline) {
+                return;
+            }
+            m_isOnline = true;
+            Q_EMIT isOnlineChanged(true);
+        });
     } else {
         NeoChatConfig::self()->setActiveConnection(QString());
     }
@@ -557,6 +592,14 @@ void Controller::setActiveConnection(Connection *connection)
     Q_EMIT activeConnectionChanged();
 }
 
+void Controller::restoreWindowGeometry(QQuickWindow *window)
+{
+    KConfig dataResource("data", KConfig::SimpleConfig, QStandardPaths::AppDataLocation);
+    KConfigGroup windowGroup(&dataResource, "Window");
+    KWindowConfig::restoreWindowSize(window, windowGroup);
+    KWindowConfig::restoreWindowPosition(window, windowGroup);
+}
+
 void Controller::saveWindowGeometry(QQuickWindow *window)
 {
     KConfig dataResource("data", KConfig::SimpleConfig, QStandardPaths::AppDataLocation);
@@ -588,7 +631,7 @@ void Controller::createRoom(const QString &name, const QString &topic)
 
 bool Controller::isOnline() const
 {
-    return m_mgr->isOnline();
+    return m_isOnline;
 }
 
 // TODO: Remove in favor of RoomManager::joinRoom
