@@ -3,7 +3,6 @@
 
 #pragma once
 
-#include <QMediaPlayer>
 #include <QObject>
 #include <QQuickItem>
 
@@ -11,7 +10,6 @@
 #include <KFormat>
 
 class QKeySequences;
-class QNetworkConfigurationManager;
 
 #include "connection.h"
 #include "csapi/list_public_rooms.h"
@@ -23,6 +21,11 @@ class NeoChatRoom;
 class NeoChatUser;
 class QQuickWindow;
 
+namespace QKeychain
+{
+class ReadPasswordJob;
+}
+
 using namespace Quotient;
 
 class Controller : public QObject
@@ -99,10 +102,11 @@ private:
     bool m_busy = false;
 
     static QByteArray loadAccessTokenFromFile(const AccountSettings &account);
-    QByteArray loadAccessTokenFromKeyChain(const AccountSettings &account);
+    QKeychain::ReadPasswordJob *loadAccessTokenFromKeyChain(const AccountSettings &account);
 
     void loadSettings();
     void saveSettings() const;
+    bool m_isOnline = true;
 
     KAboutData m_aboutData;
     bool hasWindowSystem() const;
@@ -135,13 +139,10 @@ Q_SIGNALS:
 
 public Q_SLOTS:
     void logout(Quotient::Connection *conn, bool serverSideLogout);
-    static void playAudio(const QUrl &localFile);
     void changeAvatar(Quotient::Connection *conn, const QUrl &localFile);
     static void markAllMessagesAsRead(Quotient::Connection *conn);
+    void restoreWindowGeometry(QQuickWindow *);
     void saveWindowGeometry(QQuickWindow *);
-
-private:
-    QNetworkConfigurationManager *m_mgr;
 };
 
 // TODO libQuotient 0.7: Drop
