@@ -10,18 +10,29 @@
 DevicesModel::DevicesModel(QObject *parent)
     : QAbstractListModel(parent)
 {
-    GetDevicesJob *job = Controller::instance().activeConnection()->callApi<GetDevicesJob>();
-    connect(job, &BaseJob::success, this, [this, job]() {
-        beginResetModel();
-        m_devices = job->devices();
-        endResetModel();
-    });
+    connect(&Controller::instance(), &Controller::activeConnectionChanged, this, &DevicesModel::fetchDevices);
+
+    fetchDevices();
+}
+
+void DevicesModel::fetchDevices()
+{
+    if (Controller::instance().activeConnection()) {
+        auto job = Controller::instance().activeConnection()->callApi<GetDevicesJob>();
+        connect(job, &BaseJob::success, this, [this, job]() {
+            beginResetModel();
+            m_devices = job->devices();
+            endResetModel();
+        });
+    }
 }
 
 QVariant DevicesModel::data(const QModelIndex &index, int role) const
 {
-    if (index.row() < 0 || index.row() >= rowCount(QModelIndex()))
-        return QVariant();
+    if (index.row() < 0 || index.row() >= rowCount(QModelIndex())) {
+        return {};
+    }
+
     switch (role) {
     case Id:
         return m_devices[index.row()].deviceId;
@@ -33,7 +44,7 @@ QVariant DevicesModel::data(const QModelIndex &index, int role) const
         if (m_devices[index.row()].lastSeenTs)
             return *m_devices[index.row()].lastSeenTs;
     }
-    return QVariant();
+    return {};
 }
 
 int DevicesModel::rowCount(const QModelIndex &parent) const
@@ -62,9 +73,9 @@ void DevicesModel::logout(int index, const QString &password)
             authData["identifier"] = identifier;
             auto *innerJob = Controller::instance().activeConnection()->callApi<NeochatDeleteDeviceJob>(m_devices[index].deviceId, authData);
             connect(innerJob, &BaseJob::success, this, [this, index]() {
-                Q_EMIT beginRemoveRows(QModelIndex(), index, index);
+                beginRemoveRows(QModelIndex(), index, index);
                 m_devices.remove(index);
-                Q_EMIT endRemoveRows();
+                endRemoveRows();
             });
         }
     });
@@ -74,12 +85,12 @@ void DevicesModel::setName(int index, const QString &name)
 {
     auto job = Controller::instance().activeConnection()->callApi<UpdateDeviceJob>(m_devices[index].deviceId, name);
     QString oldName = m_devices[index].displayName;
-    Q_EMIT beginResetModel();
+    beginResetModel();
     m_devices[index].displayName = name;
-    Q_EMIT endResetModel();
-    connect(job, &BaseJob::failure, this, [=]() {
-        Q_EMIT beginResetModel();
+    endResetModel();
+    connect(job, &BaseJob::failure, this, [this, index, oldName]() {
+        beginResetModel();
         m_devices[index].displayName = oldName;
-        Q_EMIT endResetModel();
+        endResetModel();
     });
 }
