@@ -46,6 +46,8 @@ QHash<int, QByteArray> MessageEventModel::roleNames() const
     roles[ShowSectionRole] = "showSection";
     roles[ReactionRole] = "reaction";
     roles[IsEditedRole] = "isEdited";
+    roles[SourceRole] = "source";
+    roles[MimeTypeRole] = "mimeType";
     roles[FormattedBodyRole] = "formattedBody";
     return roles;
 }
@@ -423,8 +425,6 @@ QVariant MessageEventModel::data(const QModelIndex &idx, int role) const
             const KFormat format;
             return format.formatRelativeDateTime(eventDate, QLocale::ShortFormat);
         }
-        case SpecialMarksRole:
-            return EventStatus::Hidden;
         }
         return {};
     }
@@ -458,7 +458,7 @@ QVariant MessageEventModel::data(const QModelIndex &idx, int role) const
         return m_currentRoom->eventToString(evt);
     }
 
-    if (role == Qt::ToolTipRole) {
+    if (role == SourceRole) {
         return evt.originalJson();
     }
 
@@ -545,6 +545,15 @@ QVariant MessageEventModel::data(const QModelIndex &idx, int role) const
         return e->content()->fileInfo()->mimeType.iconName();
     }
 
+    if (role == MimeTypeRole) {
+        auto e = eventCast<const RoomMessageEvent>(&evt);
+        if (!e || !e->hasFileContent()) {
+            return QVariant();
+        }
+
+        return e->content()->fileInfo()->mimeType.name();
+    }
+
     if (role == SpecialMarksRole) {
         if (isPending) {
             return pendingIt->deliveryStatus();
