@@ -276,10 +276,11 @@ void NeoChatRoom::saveViewport(int topIndex, int bottomIndex)
     setLastDisplayedEvent(maxTimelineIndex() - bottomIndex);
 }
 
-QVariantList NeoChatRoom::getUsers(const QString &keyword) const
+QVariantList NeoChatRoom::getUsers(const QString &keyword, int limit) const
 {
     const auto userList = users();
     QVariantList matchedList;
+    int count = 0;
     for (const auto u : userList) {
         if (u->displayname(this).contains(keyword, Qt::CaseInsensitive)) {
             NeoChatUser user(u->id(), u->connection());
@@ -289,6 +290,10 @@ QVariantList NeoChatRoom::getUsers(const QString &keyword) const
                                     {QStringLiteral("color"), user.color()}};
 
             matchedList.append(QVariant::fromValue(userVariant));
+            count++;
+            if (count == limit) { // -1 is infinite
+                break;
+            }
         }
     }
 
@@ -331,7 +336,11 @@ QString NeoChatRoom::eventToString(const RoomEvent &evt, Qt::TextFormat format,
     const bool prettyPrint = (format == Qt::RichText);
 
     using namespace Quotient;
+#ifdef QUOTIENT_07
+    return switchOnType(
+#else
     return visit(
+#endif
         evt,
         [this, prettyPrint, removeReply](const RoomMessageEvent &e) {
             using namespace MessageEventContent;
@@ -500,9 +509,9 @@ void NeoChatRoom::changeAvatar(const QUrl &localFile)
 #endif
         connect(job, &BaseJob::success, this, [this, job] {
 #ifdef QUOTIENT_07
-            connection()->callApi<SetRoomStateWithKeyJob>(id(), "m.room.avatar", localUser()->id(), QJsonObject{{"url", job->contentUri().toString()}});
+            connection()->callApi<SetRoomStateWithKeyJob>(id(), "m.room.avatar", QString(), QJsonObject{{"url", job->contentUri().toString()}});
 #else
-            connection()->callApi<SetRoomStateWithKeyJob>(id(), "m.room.avatar", localUser()->id(), QJsonObject{{"url", job->contentUri()}});
+            connection()->callApi<SetRoomStateWithKeyJob>(id(), "m.room.avatar", QString(), QJsonObject{{"url", job->contentUri()}});
 #endif
         });
     }
