@@ -11,9 +11,10 @@
 #include "knotifications_version.h"
 #include <KLocalizedString>
 #include <KNotification>
-#if KNOTIFICATIONS_VERSION >= QT_VERSION_CHECK(5, 81, 0)
-#include <KNotificationReplyAction>
+#ifdef HAVE_WINDOWSYSTEM
+#include <KWindowSystem>
 #endif
+#include <KNotificationReplyAction>
 
 #include "controller.h"
 #include "neochatconfig.h"
@@ -65,6 +66,9 @@ void NotificationsManager::postNotification(NeoChatRoom *room,
     connect(replyAction.get(), &KNotificationReplyAction::replied, this, [room, replyEventId](const QString &text) {
         room->postMessage(text, room->preprocessText(text), RoomMessageEvent::MsgType::Text, replyEventId, QString());
     });
+
+    notification->setHint(QStringLiteral("x-kde-origin-name"), room->localUser()->id());
+
     notification->setReplyAction(std::move(replyAction));
 
     notification->sendEvent();
@@ -85,6 +89,9 @@ void NotificationsManager::postInviteNotification(NeoChatRoom *room, const QStri
     notification->setPixmap(img);
     notification->setDefaultAction(i18n("Open this invitation in NeoChat"));
     connect(notification, &KNotification::defaultActivated, this, [=]() {
+#if defined(HAVE_WINDOWSYSTEM) && KNOTIFICATIONS_VERSION >= QT_VERSION_CHECK(5, 90, 0)
+        KWindowSystem::setCurrentXdgActivationToken(notification->xdgActivationToken());
+#endif
         RoomManager::instance().enterRoom(room);
         Q_EMIT Controller::instance().showWindow();
     });
@@ -95,6 +102,9 @@ void NotificationsManager::postInviteNotification(NeoChatRoom *room, const QStri
     connect(notification, &KNotification::action2Activated, this, [room]() {
         RoomManager::instance().leaveRoom(room);
     });
+
+    notification->setHint(QStringLiteral("x-kde-origin-name"), room->localUser()->id());
+
     notification->sendEvent();
     m_notifications.insert(room->id(), notification);
 }
