@@ -12,7 +12,7 @@
 
 using namespace Quotient;
 
-class RoomType : public QObject
+class NeoChatRoomType : public QObject
 {
     Q_OBJECT
 
@@ -23,6 +23,7 @@ public:
         Direct,
         Normal,
         Deprioritized,
+        Space,
     };
     Q_ENUM(Types)
 };
@@ -69,8 +70,10 @@ public:
     [[nodiscard]] QHash<int, QByteArray> roleNames() const override;
 
     Q_INVOKABLE [[nodiscard]] static QString categoryName(int section);
+    Q_INVOKABLE [[nodiscard]] static QString categoryIconName(int section);
     Q_INVOKABLE void setCategoryVisible(int category, bool visible);
     Q_INVOKABLE [[nodiscard]] bool categoryVisible(int category) const;
+    Q_INVOKABLE [[nodiscard]] int indexForRoom(NeoChatRoom *room) const;
 
     [[nodiscard]] int notificationCount() const
     {
